import os
from pathlib import Path

import click

from hierarchy import clone_hierarchy
from hierarchy.utils import display

HOME = Path(os.path.expanduser('~'))

DEFAULT_HIERARCHY_FILE = HOME / '.hierarchy'


@click.command()
@click.option('-f', '--file', 'hierarchy_file', type=click.Path(exists=True))
def main(hierarchy_file):
    if hierarchy_file:
        hierarchy_file = Path(hierarchy_file)
    else:
        hierarchy_file = DEFAULT_HIERARCHY_FILE

    display('Running Hierarchy')
    display('-----------------')
    display('')

    clone_hierarchy(hierarchy_file)


if __name__ == '__main__':
    main()



import os
from pathlib import Path

import yaml


class RepoToClone:
    url: str
    path: Path
    name: str

    def __init__(self, url: str, root_path: Path, name: str = None):
        self.url = url
        if not name:
            name = self._extract_name_from_url()
        self.path = root_path / name

    def _extract_name_from_url(self):
        return self.url.split('/')[-1].replace('.git', '')

    def __repr__(self) -> str:
        return f'<url: {self.url} | path: {self.path}>'

    def __eq__(self, other):
        return self.url == other.url and self.path == other.path


def load_from_file(hierarchy_file: Path):
    if not hierarchy_file.exists():
        raise ValueError(f'File {hierarchy_file} does not exist!')

    hierarchy_yaml = yaml.safe_load(hierarchy_file.read_text())

    _validate(hierarchy_yaml)

    hierarchy = []
    for repo_yaml in hierarchy_yaml:
        path = Path(os.path.expanduser(repo_yaml['path']))
        repo_to_clone = RepoToClone(repo_yaml['url'], path, repo_yaml.get('name'))
        hierarchy.append(repo_to_clone)

    return hierarchy


def _validate(hierarchy_yaml):
    def raise_error():
        raise ValueError('Invalid Hierarchy. Please check Hierarchy file!')

    if not hierarchy_yaml:
        raise_error()

    for repo_yaml in hierarchy_yaml:
        if 'url' not in repo_yaml or 'path' not in repo_yaml:
            raise_error()



import git

from hierarchy.loader import RepoToClone
from hierarchy.utils import display


def clone_all(repo_hierarchy: [RepoToClone]):
    for repo in repo_hierarchy:
        display(f"Cloning '{repo.url}' in '{repo.path}'")
        display('')

        git.Repo.clone_from(url=repo.url, to_path=repo.path)



from setuptools import find_packages, setup


def readme():
    with open('README.md') as f:
        return f.read()


setup(name='hierarchy',
      version='0.2.0',
      description='Clone and Maintain an entire hierarchy of Git repositories in one command',
      long_description=readme(),
      long_description_content_type="text/markdown",
      keywords='git batch',
      classifiers=[
              'Environment :: Console',
              'Intended Audience :: Developers',
              'License :: OSI Approved :: MIT License',
              'Topic :: Utilities'
      ],
      url='https://github.com/FlorianKempenich/Hierarchy',
      author='Florian Kempenich',
      author_email='Flori@nKempenich.com',
      packages=find_packages(where="src"),
      package_dir={"": "src"},
      license='MIT',
      scripts=['bin/hierarchy'],
      install_requires=[
              'click',
              'gitpython',
              'pyyaml'
      ],
      include_package_data=True)
